/*
 * ******************************************************************************************
 * File		: main.c
 * Author 	: GowinSemicoductor
 * Chip		: AE350_SOC
 * Function	: Main functions
 * ******************************************************************************************
 */

// Includes ---------------------------------------------------------------------------------
#include "Driver_UART.h"
#include "ram.h"
#include "uart.h"
#include "delay.h"
#include <stdio.h>


// Definitions ------------------------------------------------------------------------------

extern AE350_DRIVER_UART Driver_UART3;		// UART3

static volatile char uart_event_complete = 0;

static void wait_complete(void)
{
	while (!uart_event_complete);
	uart_event_complete = 0;
}

// UART callback event
void myUART_callback(uint32_t event)
{
    switch (event)
    {
    	case AE350_UART_EVENT_RECEIVE_COMPLETE:
    	case AE350_UART_EVENT_TRANSFER_COMPLETE:
    	case AE350_UART_EVENT_SEND_COMPLETE:
    	case AE350_UART_EVENT_TX_COMPLETE:
    		/* Success: continue task */
    		uart_event_complete = 1;
    		break;

    	case AE350_UART_EVENT_RX_TIMEOUT:
    		while(1);
    		break;

    	case AE350_UART_EVENT_RX_OVERFLOW:
    	case AE350_UART_EVENT_TX_UNDERFLOW:
    		while(1);
    		break;
    	default:
    		while(1);
    		break;
    }
}

// Application entry function
int main(void)
{
	//----------------------------------- 1 ------------------------------------------//
    uart_init(38400);	// Initializes UART3

    printf("\r\nIt's an APB to APB 16 Bridge (Slave 1 and Slave 2) demo.\r\n");
    printf("These are UART3 printf() and RAM demos.\r\n");

    for(int i = 0; i < 1000; i++)
    {
    	printf("\r\n- - - - - - - - - - - - - - - - - - - - - - - - - - - \r\n");
    	printf("Loop %d\r\n", i);

    	// Write data into RAM
    	printf("Write data into RAM...\r\n");
    	setDataCmd(3);		// Command: select and write
    	setDataAddr(i);		// Address
    	setDataIn(230+i);	// Data input

    	// Read data from RAM
    	printf("Read data from RAM...\r\n");
    	setDataCmd(2);		// Command: select and read
    	printf("Address    : %d\r\n",getDataAddr());
    	printf("Data input : %d\r\n",getDataIn());
    	printf("Data output: %d\r\n",getDataOut());

    	simple_delay_ms(1000);
    }


    printf("\r\n\r\n\r\n");
    printf("\r\n*************************************************************************\r\n");
    printf("*************************************************************************\r\n");
    printf("*************************************************************************\r\n");
    printf("\r\n\r\n\r\n");


    //------------------------------------- 2 ----------------------------------------//
    static AE350_DRIVER_UART *UARTdrv = &Driver_UART3;	//UART3
    char cmd = 0;

    /* Initializes the UART driver */
    UARTdrv->Initialize(myUART_callback);

    /* Power up the UART peripheral */
    UARTdrv->PowerControl(AE350_POWER_FULL);

    /* Configure the UART to 38400 Bits/second */
    UARTdrv->Control(AE350_UART_MODE_ASYNCHRONOUS |			// Mode			: asynchronous
    				 AE350_UART_DATA_BITS_8 |				// Data bits	: 8-bit
					 AE350_UART_PARITY_NONE |				// Parity		: none
					 AE350_UART_STOP_BITS_1 |				// Stop bit:    : 1-bit
					 AE350_UART_FLOW_CONTROL_NONE, 38400);	// Flow control : none
    														// Baud rate    : 38400

    /* Enable Receiver and Transmitter lines */
    UARTdrv->Control (AE350_UART_CONTROL_TX, 1);			// Transmit
    UARTdrv->Control (AE350_UART_CONTROL_RX, 1);			// Received

    UARTdrv->Send("\r\nIt's an APB to APB 16 Bridge (Slave 2) demo.\r\n", 46);
    wait_complete();

    UARTdrv->Send("\r\nThis is UART3 received demo.\r\n", 30);
    wait_complete();

    UARTdrv->Send("\r\nInput 'G' to receive a message", 34);
    wait_complete();

    // Input character "G"
    // Print "Hello World!"
    while (1)
    {
    	// Receive 'G'
        UARTdrv->Receive(&cmd, 1);           /* Get a character 'G' from UART */
        wait_complete();
        if (cmd == 'G')                      /* 'G', send greeting  */
        {
        	UARTdrv->Send("\r\nHello World!", 14);
        	wait_complete();
        }
    }

    return 0;
}
