/*
 * ******************************************************************************************
 * File		: ram.h
 * Author	: GowinSemicoductor
 * Chip		: AE350_SOC
 * Function	: RAM definitions.
 * ******************************************************************************************
 */

#ifndef __RAM_H__
#define __RAM_H__

// Includes ---------------------------------------------------------------------------------
#include "platform.h"


// Definitions ------------------------------------------------------------------------------

// Register definition
typedef struct
{
	__IO   unsigned int  DCMD;			/* Offset: 0x000 (port select) 	 [15:0] */
	__IO   unsigned int  DADDR;			/* Offset: 0x004 (write or read) [15:0] */
	__IO   unsigned int  DIN;			/* Offset: 0x008 (write only)    [15:0] */
	__I    unsigned int  DOUT;			/* Offset: 0x00C (ready only)    [15:0] */
} RAM_RegDef;


#define RAM_BASE				_IO_(APB_EXT_BASE + 0x400000)	// RAM address: slave 1
#define AE350_RAM				((RAM_RegDef *) RAM_BASE )		// RAM register definition
#define DEV_RAM					AE350_RAM						// Device RAM

// Bit definitions
#define DATACMD		   			((unsigned int) 0x00000003)
#define DATAADDR	   			((unsigned int) 0x0000007F)
#define DATAIN				 	((unsigned int) 0x0000FFFF)
#define DATAOUT					((unsigned int) 0x0000FFFF)


// Declarations ----------------------------------------------------------------------------

void setDataCmd(unsigned int cmd);		// Set data command
unsigned int getDataCmd(void);			// Get data command

void setDataAddr(unsigned int addr);	// Set data address
unsigned int getDataAddr(void);			// Get data address

void setDataIn(unsigned int din);		// Set input data
unsigned int getDataIn(void);			// Get input data

unsigned int getDataOut(void);			// Get output data


#endif	/* __RAM_H__ */
